#!/usr/bin/python 

import os, sys
import pickle
import itertools as itt
import operator as op
import numpy as np
from numpy import ma
from scipy import stats, ndimage, sparse
from scipy.interpolate import splev

import matplotlib as mpl
import pylab as pl

from imfun import fseq, fnmap, lib, filt, cluster, bwmorph
from swan import pycwt
from swan.gui import swanrgb

dtype = np.float32
lib._maxshape_ = 1e10
pycwt._maxshape_ = 1e10
fseq._dtype_ = np.float32


mpl.rcParams['font.size'] = 10.0

imresize = lib.imresize
fps = 25

def in_range(v, (low,high)):
    if low > high:
	low,high=high,low
    return (v >= low)*(v < high)

def eds2weights(eds):
    weights = np.zeros(eds.shape, dtype)
    for column in range(eds.shape[1]):
	weights[:,column] = lib.rescale(eds[:,column])
    return weights

def phase2(freqs, f_s, subsample=1):
    def _fn(v):
	v = lib.DFoSD(v)
        if subsample > 1:
            v = filt.gauss_smooth(v, sigma = subsample*2)[::subsample]
	cwt = pycwt.cwt_f(v, freqs, f_s)
	phase = pycwt.cwt_phase(cwt)
	weights = eds2weights(pycwt.eds(cwt))
	return np.average(pycwt.cwt_phase(cwt), axis=0,
			  weights = weights)
    return _fn


def getfreqs(frange, nfreqs=200):
    return np.linspace(frange[0], frange[1], nfreqs)


_resize = True

imkw = {'aspect':'equal', 'interpolation':'nearest', 'origin':'lower'}

frange_tgf = (0.01, 0.04)
#frange_myo = (0.09, 0.19)

def make_mask(seq, nfirst=20, lowpercentile=15):
    frames = lib.take(20, seq.frames())
    avf = np.mean(frames, axis=0)
    low = stats.scoreatpercentile(avf.ravel(), lowpercentile)
    return avf > low


def try_load_resized(fname, suffix='-resized.npy'):
    dataname = fname+suffix
    seq = fseq.FSeq_hdf5(fname,dt=1.0)
    dt = seq.dt    
    if os.path.exists(dataname):
        data = np.load(dataname)
        L = data.shape[0]
        return fseq.open_seq(data, dt=dt*seq.length()/float(L))
    else:
	print "Can't locate the resized movie, resizing..."
        key = '%d hz'%round(1/dt)
        print key, "in resize_kw?", key in resize_kw
        if key in resize_kw:
            kw = resize_kw[key]
        else:
            kw = resize_kw['25 hz']
        print "resizing as...", kw
        sys.stdout.flush()
        seqr = resize_seq(fname, **kw)
        np.save(dataname, seqr.data)
        return seqr
    
def resize_seq(fname, sdown=1, tdown = 20, ssigma=1.0, tsigma=1.0):
    seq = fseq.FSeq_hdf5(fname)
    dt = 1.0 
    tsig = tsigma / dt # tsigma  ~ 1 sec
    arr = seq.as3darray(dtype='float32') # do all filtering in float32
    out = ndimage.gaussian_filter(arr, sigma = [tsig, ssigma, ssigma])[::tdown,::sdown,::sdown]
    return fseq.open_seq(out, dt=dt*tdown)
  

resize_kw = {'25 hz': dict(sdown=1, tdown=20, ssigma=1., tsigma=1.),
             '1 hz': dict(sdown=2, tdown=1, ssigma=3., tsigma=0.1)}
    
ffmpeg_template = "ffmpeg -i %s.avi -r %f -f mp4 %s.mp4"
mencoder_template = "mencoder 'mf://_tmpx-%s*.png' -fps %f -o %s.avi"
mencoder_opts = {
    'x264': mencoder_template + ' -ovc x264',
    'mpeg4': mencoder_template+' -ovc lavc -lavcopts vcodec=mpeg4',
    'mjpeg': mencoder_template+' -ovc lavc -lavcopts vcodec=mjpeg',
    'theora': mencoder_template+' -ovc lavc -lavcopts vcodec=libtheora',
    'mpeg2':mencoder_template+' -ovc lavc -lavcopts vcodec=mpeg2video',
    #'xvid': mencoder_template+' -ovc lavc -lavcopts vcodec=libxvid',
    'x264-other':mencoder_template+' -ovc lavc -lavcopts vcodec=libx264',
    #'xvid-other':mencoder_template+' -ovc xvid',
    'no': mencoder_template+' -ovc copy',
    }


def locmax_pos(m):
    mfreqs,mtimes = np.where(np.diff(np.sign(np.diff(m, axis=0)), axis=0) < 0)
    i = np.argsort(mtimes)
    return mtimes[i], mfreqs[i]+2
    

def dominant_freq(freqs, f_s, subsample=1, psi=pycwt.Morlet(), variant=2):
    f0,f1 = freqs[0], freqs[-1]
    fsd = psi.sigma_f_f(np.mean(freqs))
    n = 3
    rhsd = n*fsd
    dfr = np.diff(freqs).mean()
    rhsd_i = int(round(fsd/dfr))
    e_threshold = 1.5
    min_pts_ridge = 100
    def _fn(v):
	v = lib.DFoSD(v)
        if subsample > 1:
            v = filt.gauss_smooth(v, sigma = subsample*2)[::subsample]
	L = len(v)
	tstop = L/float(f_s)
	tvec = np.arange(0, tstop + f_s, 1./float(f_s))[:L]

        cwt = pycwt.cwt_f(v, freqs, f_s,wavelet=psi)
	eds = pycwt.eds(cwt,f0=psi.f0)
        
	xt,xf = locmax_pos(eds)
	f = np.zeros(L)
	if variant ==1:
	    avg_end = min(500, L)
	    seed = lib.locextr(eds[:,:avg_end].mean(axis=1),x=freqs,
			       output='max')[0][0]
	    xe = eds[xf, xt]
	    for i,t in enumerate(tvec):
		indt = xt==t
		if not np.any(indt):
		    continue
		ft,et = xf[indt], xe[indt]
		ind = np.abs(freqs[ft]-seed)<=rhsd
		## if there is an 'old' ridge nearby, prefer its neighborhood
		if np.any(ind):
		    f[i] = freqs[ft[ind][np.argmax(et[ind])]]
		## otherwise, pick the highest 
		else: 
		    f[i] = freqs[ft[np.argmax(et)]]
		seed = f[i]
	elif variant==2:
	    D = np.zeros(eds.shape)
	    D[xf,xt] = 1
	    s = ndimage.generate_binary_structure(2,2)
	    D = 1.0*ndimage.binary_dilation(D)
	    labels, nlab = ndimage.label(D, s)
	    objs = ndimage.find_objects(labels)
	    for mark,obj in enumerate(objs):
		m = labels == mark+1
		if m.sum() < min_pts_ridge:
		    D[m] = 0
		    continue
		tsl = obj[1]
		t_ext = tsl.stop - tsl.start
		D[m] = t_ext * eds[m].mean()
            f_i = np.argmax(D, axis=0)
	    f = freqs[f_i]
            phase = pycwt.cwt_phase(cwt)
            savg = np.array([phase[f_i[k]-rhsd_i : f_i[k]+rhsd_i+1,k].mean()
                             for k in range(L)])
            savg[np.isnan(savg)] = 0 
	return f + savg*1j # keep frequencies in real, phase in imaginary
    return _fn


def make_avg_fft(fname, seq=None, max_pixels=None,
		 verbose=True,
		 rand=True,
		 sq_size=100,
		 max_freq =0.25,
		 _pow = 2,
		 tag = '',
		 make_plots=True):
    if seq is None:
	if _resize:
	    seq = try_load_resized(fname)
	    seq0 = fseq.FSeq_hdf5(fname,dt=1.0)
	else:
	    seq = fseq.FSeq_dfh5(fname,dt=1.0)
    print fname, seq.length(), seq.shape()
    mask=make_mask(seq)
    npix = np.sum(mask)
    print "%d pixels in the mask"%npix
    max_pixels = lib.ifnot(max_pixels,npix)
    pixel_counter = 0
    npix = min(npix, max_pixels)
    L = seq.length()
    Lhalf = L/2
    freqs = np.fft.fftfreq(int(L), seq.dt)[:Lhalf]
    out = np.zeros(freqs.shape)
    out_map = np.zeros(mask.shape)
    if not rand:
	squares = cluster.make_grid(mask.shape, sq_size)
	seq.fns = [fseq.identity] #hack to use generic pix_iter	
    else:
	squares = [(slice(0,mask.shape[0]), slice(0, mask.shape[1]))]
	seq.fns = [] # hack to use FSeq_MLF pix_iter
    keepf = in_range(freqs, frange_tgf)
    for sq in squares:
	r0,c0 = sq[0].start, sq[1].start
	pixel_iter = seq.pix_iter(mask=mask,rand=rand,sliceobj=sq)
	for s,i,j in pixel_iter:
	    s = lib.DFoSD(s)
	    y = (np.abs(np.fft.fft(s))[:Lhalf])**_pow
	    out += y
	    out_map[r0+i,c0+j] = np.mean(y[keepf])
	    pixel_counter+= 1
	    if verbose and not (pixel_counter%10):
		sys.stderr.write("\rpixel %05d of %05d"%(pixel_counter,npix))
	    if pixel_counter > max_pixels:
		break
    print '\n'
    out /= pixel_counter
    print out_map.min(), out_map.max()
    newfreqs=freqs[freqs<max_freq][1:]
    newy = out[freqs<max_freq][1:]
    if len(newy)>20:
	miny = np.min(newy[-20:])
    else:
	miny = np.min(newy[5:])
    if make_plots:
	pl.figure()
	pl.plot(newfreqs, newy, 'k-')
	pl.ylim([miny, np.min([np.max(out),150])])
	pl.axvspan(*frange_tgf, alpha=0.5, color='lightyellow')
	pl.savefig(fname+'-avg-fft.png')
	pl.figure()
	pl.imshow(out_map, aspect='equal', cmap='spectral')
	pl.colorbar()
	pl.savefig(fname+'-avg-fft-map-tgf.png')
	np.save(fname+'-%s-avg-fft-map-tgf.npy'%tag,out_map)
	np.save(fname+'-%s-avg-fft-spectrum-tgf.npy'%tag,np.array([newfreqs,newy]))
    return newfreqs,newy,out_map
    
def make_avg_spectrogram(fname, seq = None,
			 frange = None,
			 nfreqs = 256,
			 max_pixels=None, verbose=True,
			 rand=True,
			 sq_size=64,
			 wavelet = pycwt.Morlet(),
			 ):
    if frange is None:
	frange = [frange_tgf[0]-frange_tgf[0]/2,
		  frange_tgf[1]+frange_tgf[1]/2]
    if seq is None:
	seq = fseq.open_seq(fname)
    if not rand:
	seq.fns = [fseq.identity] # hack to use generic pix_iter
    else:
	seq.fns = []
    print fname, seq.length(), seq.shape()
    mask=make_mask(seq)
    out = np.zeros((nfreqs,seq.length()))
    freqs = np.linspace(frange[0], frange[1], nfreqs)
    squares = cluster.make_grid(mask.shape, sq_size)
    Fs = 1./seq.dt
    pixel_counter = 0
    npix = np.sum(mask)
    max_pixels = lib.ifnot(max_pixels,npix)
    for sq in squares:
	r0,c0 = sq[0].start, sq[1].start
	pixel_iter = seq.pix_iter(mask=mask,rand=rand,sliceobj=sq)
	for s,i,j in pixel_iter:
	    s = lib.DFoSD(s)
	    eds = pycwt.eds(pycwt.cwt_f(s, freqs, Fs, wavelet, 'zpd'))
	    out += eds
	    pixel_counter += 1
	    if verbose and not (pixel_counter%10):
		sys.stderr.write("\rpixel %05d of %05d"%(pixel_counter,npix))
	    if pixel_counter > max_pixels:
		print 'max pixel count reached'
		break
    print '\n'
    out /= pixel_counter
    np.save(fname+'-avg-spectrogram-tgf.npy',out)
    pl.figure()
    extent=[0, seq.dt*seq.length(), frange[0],frange[1]]
    pl.imshow(out, cmap="spectral",extent=extent); pl.colorbar()
    max_pos = fnmap.loc_max_pos
    t = np.arange(0, seq.length()*seq.dt,seq.dt)
    for j in range(seq.length()):
                    locmax = np.array(max_pos(out[:,j]))
                    asort = np.argsort(out[locmax,j])[::-1]
                    locmax = locmax[asort][:4]
                    pl.plot([t[j]]*len(locmax), freqs[locmax],'k,')
    pl.axis(extent)
    pl.savefig(fname+'-avg-spectrogram.png')
    pl.close()
    return out
    
def make_mapping_movie(fname,
                       seq = None,
		       cmap=swanrgb,
		       type="freq",
                       make_snapshots=False):
    func_options = dict(phase=phase2,
			freq=dominant_freq)
    fn = func_options[type]
    tag = '-' + '-'.join((type, 'tgf','ver2'))

    if seq is None: # only have filename
        if _resize:
            seq= try_load_resized(fname)
            tag += '-resized'
        else:
            seq = fseq.FSeq_hdf5(fname)

    print fname, seq.length(), seq.shape(), seq.dt
    sys.stdout.flush()
    subsample = 1
    
    mask = make_mask(seq)
    sp = seq.pw_transform(fn(getfreqs(frange_tgf),
			     1.0/seq.dt,
			     subsample=subsample),
			  verbose=True,
			  dtype = 'complex',
			  mask=mask)
    print sp.length(), sp.dt, 1/sp.dt
    np.save(fname+tag, sp.data)
    trio_movie2(seq, sp, fname)
    L = sp.length()
    frames_gen = sp.frames()
    if make_snapshots:
        for i in xrange(0,L-500,500):
            frames = [np.ma.masked_where(1-mask,np.real(f)) for f in sp.data[i:i+500:10]]
            lib.group_maps(frames,10,range(i,i+500,10), samerange=True,imkw=imkw)
            pl.savefig(fname+'-%s-%04d.png'%(tag,i))
            pl.close()

def trio_movie2(seq0, seqp, aviname, mask=None,suff='trio',speedupFactor=20):
    from matplotlib import animation
    if mask is None:
        mask = make_mask(seq0)
    mfn = lambda f: np.ma.masked_where(-mask, f)
    seq0.fns = [mfn] # TODO: append?
    seqp.fns = [mfn]
    ranges = [seq0.data_percentile((0.1, 99.9)), 
             np.percentile(seqp.data.real, (0.1, 99.9))]
    
    imkw1 = dict(cmap='gray', aspect='equal', 
                 vmin=ranges[0][0], vmax=ranges[0][1])
    imkw2 = dict(cmap='spectral', aspect='equal',
                 vmin = ranges[1][0], vmax=ranges[1][1])
    imkw3 = dict(cmap=pl.cm.gist_rainbow, aspect='equal',
                 vmin = -np.pi, vmax = np.pi)
                 
    #pl.figure(figsize=(12,4)); pl.hold('off')
    fh, axs = pl.subplots(1,3,figsize=(12,4),
                          sharex=True, sharey = True)

    supt = fh.suptitle("")
    pl.hold('off')
    fnames = []
    L = seqp.length()
    print 'Sequence length: ', L
    sh = seqp.shape()
    plots = []
    def _init():
        k = 0
        plots.append(axs[0].imshow(seq0[k],**imkw1))
        plots.append(axs[1].imshow(mfn(seqp.data[k].real), **imkw2))
        plots.append(axs[2].imshow(mfn(seqp.data[k].imag), **imkw3))
        pl.setp(axs[0], title='LSI',
                xlim = (0, sh[1]),
                ylim = (0, sh[0]))
        for ax in axs:
            pl.setp(ax, frame_on=False, xticks = [], yticks=[])
        pl.setp(axs[1], title='frequency')
        pl.setp(axs[2], title='phase')
        supt.set_text('time %03.1f'%(k*seqp.dt))
        return plots

    def _animate(k):
        sys.stderr.write('\r k is %04d '%k)
        plots[0].set_data(seq0[k])
        plots[1].set_data(mfn(seqp.data[k].real))
        plots[2].set_data(mfn(seqp.data[k].imag))
        #pl.setp(axs[0], xlim = (0, sh[1]), ylim = (0, sh[0]))
        supt.set_text('time %03.1f'%(k*seqp.dt))
        return plots

    anim = animation.FuncAnimation(fh, _animate, init_func=_init, frames=L,)        
    mencoder_extra_args=['-ovc', 'x264', '-really-quiet']
    pl.close(anim._fig)
    outname = '-'.join([aviname, suff]) + '.mp4'
    Writer = animation.writers.avail['avconv']
    anim.save(outname, writer=Writer(fps=speedupFactor/seqp.dt,bitrate=1800), extra_args=mencoder_extra_args)
    return anim

def trio_movie(resized, freqmaps, phasemaps, aviname, dt=1,suff='test'):
    fseqs = map(fseq.open_seq, [resized, freqmaps, phasemaps])
    ranges = [fs.data_percentile((0.5, 99.5)) for fs in fseqs]
    pl.figure(figsize=(12,4)); pl.hold('off')
    fnames = []
    k = 1
    mencoder_command = mencoder_opts['x264']%(suff,fps,aviname)
    print mencoder_command
    L = len(resized)
    for trio in itt.izip(*[fs.frames() for fs in fseqs]):
	if not k%10 :
	    sys.stderr.write('\r%04d out of %04d (%02.2f%%)'\
			     %(k,L,round(100*float(k)/L,2)))
	pl.subplot(131); pl.title('flow')
	pl.imshow(trio[0], vmin=ranges[0][0],vmax=ranges[0][1],cmap='gray',aspect='equal')
	pl.subplot(132); pl.title('frequencies')
	pl.imshow(trio[1], vmin=ranges[1][0],vmax=ranges[1][1],cmap='spectral',aspect='equal')
	pl.title('time: %03d s'%k)
	pl.subplot(133); pl.title('phase')
	pl.imshow(trio[2], vmin=ranges[2][0],vmax=ranges[2][1],cmap='jet',aspect='equal')
	#pl.suptitle("")
	#pl.suptitle('time: %03d s'%k)
	fnames.append('_tmpx-%s_%05d.png'%(suff,k))
	pl.setp(pl.gca(), 'xticks', [], 'yticks', [])
	k+=1
	pl.savefig(fnames[-1])
    print "\n"
    pl.close()

    os.system(mencoder_command)
    map(os.remove, fnames)
	
#

def load_fft_cif(name):
    fft_ci = pickle.load(open(name))
    return {0.4: lambda x:splev(x, fft_ci[0.4]),
            1: lambda x:splev(x, fft_ci[1])}

def preprocess_file(fname, sign_lev=None,verbose=False):
    """
    Return:
        - resized frame sequence
        - mask, which is true, where oscillations were significant // lg(P)/10 > sign_lev
        - (peak, fm), where peak is a map of Fourier power lg(P)/10  and fm is a map of the frequency of the peak
    """
    # resized
    save_path = '/home/brazhe/Dropbox/active-papers/kidney-speckle-methods/figures/'
    seqr = try_load_resized(fname)
    print "Loaded downsampled data of shape ", seqr.data.shape
    print "sampling rate: ", seqr.dt
    maskr = make_mask(seqr, 200)
    bmaskr = lib.mask4overlay(maskr<1,2, 0.5)
    
    # fft
    freqs = np.fft.fftfreq(seqr.length(), seqr.dt)
    _frange = np.array(frange_tgf) * np.array((0.75, 1.25)) # 25% margins round frange_tgf

    freqs_clip = freqs[in_range(freqs, _frange)]
    fn = lambda v: np.abs(np.fft.fft(lib.DFoSD(v))[in_range(freqs,_frange)])**2
    seqt = seqr.pw_transform(fn, verbose=verbose) ## clipped power spectra
    
    # mask where oscillations were significant
    peak = np.log10(seqt.max_project())/10
    fm = seqt.time_project(fn=lambda aa: freqs_clip[np.argmax(aa)])
    
    fft_cif = load_fft_cif(save_path+'fft_ci.pickle')
    
    if sign_lev is None:
        if seqr.dt in fft_cif:
            sign_lev = fft_cif[seqr.dt](seqr.length())
        else:
            sign_lev = fft_cif[1](seqr.length())
    
        print "Significance level is set to %2.2f" % sign_lev
    
    sig_osc = (peak > sign_lev)  # True where oscillations are significant
    sig_osc = maskr*sig_osc*filt.opening_of_closing(sig_osc) # filter out small dots
    
    maskF = bwmorph.filter_mask(sig_osc, lambda r: bwmorph.filter_size_regions(r, 25))
    maskF = ndimage.binary_fill_holes(maskF)
    return seqr, maskF, (peak, fm)

def make_vec_explained(x, nmodes=3, nt=125,dt=1):
    tvec = np.arange(nt,x.shape[1]-nt,25)
    y = np.zeros(len(tvec))
    for k,t in enumerate(tvec):
        u,s,vh = np.linalg.svd(x[:,t-nt:t+nt], full_matrices=False)
        y[k] = (np.sum(s[:nmodes]**2)/np.sum(s**2))
        sys.stderr.write('\r %03d'%t)
    return tvec*dt, y

def coll_svd_explained(flist1_c,data_path, nexplained=4):
    acc = []
    for name in flist1_c:
        print name
        seqr, maskr, (peak, fm) = preprocess_file(data_path + name)
        frmap,name = get_frmap(data_path+name)
        frmap, phasemap = frmap.real, frmap.imag
        mask1w = reduce(op.__and__, (f>0 for f in frmap))
        del frmap
        acc.append(make_vec_explained(reshape_from_masked_video(phasemap, maskr*mask1w),nexplained))
    return acc

def pc_index(ph1,ph2):
    "calculate phase coherence index"
    return np.abs(np.mean(np.exp(1j*(ph1-ph2)),axis=0))

def point_sync(phm, (r0,c0),mask1, nhood=32,is_unwrapped=False):
    "with un-wrapped phase"
    sh = phm.shape[1:]
    out = np.zeros(sh)
    
    rbounds = max(0, r0-nhood), min(sh[0],r0+nhood+1)
    cbounds = max(0, c0-nhood), min(sh[1],c0+nhood+1)
    
    if is_unwrapped:
        v = phm[:,r0,c0]
        sl = (slice(None), slice(*rbounds),slice(*cbounds))
        out[sl[1:]] = pc_index(phm[sl],v.reshape(-1,1,1))
        mask2 = np.ones(sh,'bool')
        mask2[sl[1:]]=0
        return ma.masked_where(-mask1, out) 
    v = my_unwrap(phm[:,r0, c0])
    for i in xrange(*rbounds):
        for j in xrange(*cbounds):
            if mask1[i,j]:
                v2 = my_unwrap(phm[:,i,j])
                out[i,j] = pc_index(v,v2)
    return ma.masked_where(-mask1, out)


def local_phase_index(uphmap, mask1, nhood=16):
    sh = mask1.shape
    out = np.zeros(sh)
    
    points = cluster.mask2points(mask1)
    Lp = len(points)
    for k,p in enumerate(points):
        sys.stderr.write('\rprocessing point %04d of %d'%(k, Lp))
        i,j = p
        if not mask1[p]: continue
        v = uphmap[:,i,j]
        rbounds = max(0,i-nhood), min(i+nhood+1,sh[0])
        cbounds = max(0,j-nhood), min(j+nhood+1,sh[1])
        sl = (slice(None), slice(*rbounds), slice(*cbounds))
        N = mask1[sl[1:]].sum()
        uph = uphmap[sl] - v.reshape(-1,1,1)
        m = np.abs(np.mean(np.exp(1j*uph), axis=0))
        out[i,j] = m[mask1[sl[1:]]].sum()/N
    return out

def my_unwrap(v):
    idx = np.arange(len(v))
    keep = np.diff(v) > 0
    if any(keep):
        uv = np.unwrap(v[keep])
        return np.interp(idx, idx[keep], uv)
    else:
        return np.unwrap(v)

def unwrap_phasemap(phm,mask1):
    sh = mask1.shape
    out = np.zeros(phm.shape)
    for row in xrange(sh[0]):
        for col in xrange(sh[1]):
            if mask1[row,col]:
                out[:,row,col] = my_unwrap(phm[:,row,col])
    return out

def local_coh_map(name, nhood=8):
    seqr, maskr, _ = preprocess_file(name)
    frmap,_ = get_frmap(name)
    frmap, phasemap = frmap.real, frmap.imag
    maskw = reduce(op.__and__, (f>0 for f in frmap))
    maskwr = maskw*maskr
    del frmap, seqr, maskw
    uphasemap = unwrap_phasemap(phasemap, maskwr)
    del phasemap
    out = local_phase_index(uphasemap, maskwr, nhood=nhood)
    return out

def phase_coherence_clusterize(name, lcoh_map, min_coh=0.5):
    _, maskr, _ = preprocess_file(name)
    frmap,_ = get_frmap(name)
    _, phasemap = frmap.real, frmap.imag
    maskwr = maskr*reduce(op.__and__, (f>0 for f in frmap))
    del frmap, _
    uphasemap = unwrap_phasemap(phasemap, maskwr)
    del phasemap
    locs = locate_potential_cluster_centres(lcoh_map)
    print "...loaded data "
    mlist = np.array([point_sync(uphasemap, l, maskwr, 500, is_unwrapped=True)
                      for l in locs])
    print "...calculated point-sync maps"
    out = np.zeros(maskwr.shape)
    X = cluster.mask2points(maskwr)
    for l in X:
        coh = [m[l] for m in mlist]
        if np.max(coh) > min_coh:
            out[l] = np.argmax(coh) + 1
    return out
        
def prune_labels(larr, minarea=100):
    nlabs = np.max(larr)
    out = np.zeros(larr.shape, np.int)
    lcount = 1
    for l in range(1, nlabs+1):
        loc = np.where(larr == l, True, False)
        if np.sum(loc) >= minarea:
            out[loc] = lcount
            lcount +=1
    return out, lcount -1

def locate_potential_cluster_centres(arr, coh_percentile=95):
    m = arr>np.percentile(arr, coh_percentile)
    ll, nlab = ndimage.label(m)
    fll, nlab2 = prune_labels(ll)
    mlocs = ndimage.maximum_position(arr, fll, np.arange(nlab2)+1)
    return list(np.asarray(mlocs, 'int'))


    
def make_mask1(frmap):
    return reduce(op.__and__, frmap>0)
    

def reshape_from_masked_video(mov, mask1=None):    
    if mask1 is None:
	mask1 = reduce(op.__and__, [f>0 for f in mov])
    locs = list(cluster.locations(mask1.shape))
    locs = [l for l in locs if mask1[l]]
    out = np.zeros((len(locs), len(mov)))
    for k,l in enumerate(locs):
	out[k] = mov[:,l[0],l[1]]
    return out

def reshape_to_masked_video(arr, mask1):
    sh = mask1.shape
    out = np.zeros([arr.shape[1]] + list(sh))
    locs = list(cluster.locations(mask1.shape))
    locs = [l for l in locs if mask1[l]]
    for k,l in enumerate(locs):
	out[:,l[0],l[1]] = arr[k]
    return out    
    

def get_frmap(name):
    suff = ['-freq-tgf-ver2-resized.npy', 'freq-tgf-resized.npy','freqmap-tgf-resized.npy']
    for s in suff:
        print name + s 
	if os.path.exists(name+s):
	    return np.load(name + s), name + s
